@@ -53,14 +53,26 @@ CONF = config.CONF
 # Note that it will not affect the version of the Nova API used by
 # Watcher, only the version used by tempest when used from those tests.
 NOVA_API_VERSION_CREATE_WITH_HOST = '2.74'
+# Nova API version that adds 'pinned_availability_zone' to GET /servers/details
+# response.
+NOVA_API_VERSION_SERVER_PINNED_AZ = '2.96'
 
 
 class BaseInfraOptimScenarioTest(manager.ScenarioTest):
     """Base class for Infrastructure Optimization API tests."""
 
+    # Default Goal and Strategy to be defined in each test class that inherit
+    # from this base class.
+    GOAL = None
+    STRATEGY = None
+
     min_microversion = None
     max_microversion = manager.LATEST_MICROVERSION
 
+    # Holds the initial configuration of available compute nodes, to assist
+    # the rollback procedure.
+    initial_compute_nodes_setup = []
+
     # States where the object is waiting for some event to perform a transition
     IDLE_STATES = ('RECOMMENDED', 'FAILED', 'SUCCEEDED', 'CANCELLED')
     # States where the object can only be DELETED (end of its life-cycle)
@@ -261,6 +273,12 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         return all([ap['state'] in cls.AP_FINISHED_STATES
                     for ap in action_plans['action_plans']])
 
+    def check_min_enabled_compute_nodes(self, min_nodes):
+        enabled_compute_nodes = self.get_enabled_compute_nodes()
+        msg = ("This test required at least %s enabled compute nodes, but "
+               "only %s were found." % (min_nodes, len(enabled_compute_nodes)))
+        self.assertGreaterEqual(len(enabled_compute_nodes), min_nodes, msg=msg)
+
     def wait_for_all_action_plans_to_finish(self):
         assert test_utils.call_until_true(
             func=self._are_all_action_plans_finished,
@@ -911,6 +929,11 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         _, audit = cls.client.show_audit(audit_uuid)
         return audit.get('state') in cls.AUDIT_FINISHED_STATES
 
+    @classmethod
+    def is_audit_ongoing(cls, audit_uuid):
+        _, audit = cls.client.show_audit(audit_uuid)
+        return audit.get('state') == 'ONGOING'
+
     # ### ACTION PLANS ### #
 
     def delete_action_plan(self, action_plan_uuid):
@@ -922,6 +945,10 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         resp, _ = self.client.delete_action_plan(action_plan_uuid)
         return resp
 
+    def has_action_plans(self, audit_uuid=None):
+        _, action_plans = self.client.list_action_plans(audit_uuid=audit_uuid)
+        return len(action_plans['action_plans']) > 0
+
     def has_action_plan_finished(self, action_plan_uuid):
         _, action_plan = self.client.show_action_plan(action_plan_uuid)
         return action_plan.get('state') in self.AP_FINISHED_STATES
@@ -934,23 +961,36 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
                 return False
         return True
 
-    def execute_strategy(self, goal_name, strategy_name,
-                         expected_actions=[], **audit_kwargs):
-        """Execute an action plan based on the specific strategy
+    def has_action_plans_recommended(self, audit_uuid=None):
+        _, action_plans = self.client.list_action_plans(audit_uuid=audit_uuid)
+        for ap in action_plans['action_plans']:
+            _, action_plan = self.client.show_action_plan(ap['uuid'])
+            if action_plan.get('state') == 'RECOMMENDED':
+                return True
+        return False
+
+    def create_audit_template_for_strategy(self, goal_name=None,
+                                           strategy_name=None):
+        """Create and return an audit template for the given goal/strategy."""
+
+        goal_name = goal_name or self.GOAL
+        strategy_name = strategy_name or self.STRATEGY
 
-        - create an audit template with the specific strategy
-        - run the audit to create an action plan
-        - get the action plan
-        - Verify that all action types in the expected_actions
-          list are present in the action plan.
-        - run the action plan
-        - get results and make sure it succeeded
-        """
         _, goal = self.client.show_goal(goal_name)
         _, strategy = self.client.show_strategy(strategy_name)
         _, audit_template = self.create_audit_template(
             goal['uuid'], strategy=strategy['uuid'])
 
+        return audit_template
+
+    def create_audit_and_wait(self, audit_template_uuid, **audit_kwargs):
+        """Create an audit from template and wait until it finishes.
+
+        Returns the finished audit object (dict).
+        Raises on FAILED/CANCELLED states.
+        """
+        # Ensure previous action plans are finished before
+        # creating a new audit
         self.assertTrue(test_utils.call_until_true(
             func=functools.partial(
                 self.has_action_plans_finished),
@@ -962,8 +1002,9 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         state = audit_kwargs.pop('state', None)
         interval = audit_kwargs.pop('interval', None)
         parameters = audit_kwargs.pop('parameters', None)
+
         _, audit = self.create_audit(
-            audit_template['uuid'],
+            audit_template_uuid,
             audit_type=audit_type,
             state=state,
             interval=interval,
@@ -972,7 +1013,8 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         try:
             self.assertTrue(test_utils.call_until_true(
                 func=functools.partial(
-                    self.has_audit_finished, audit['uuid']),
+                    self.has_audit_finished,
+                    audit['uuid']),
                 duration=600,
                 sleep_for=2
             ))
@@ -980,46 +1022,67 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
             self.fail("The audit has failed!")
 
         _, finished_audit = self.client.show_audit(audit['uuid'])
-        if finished_audit.get('state') in ('FAILED', 'CANCELLED', 'SUSPENDED'):
+        if finished_audit.get('state') in (
+                'FAILED', 'CANCELLED', 'SUSPENDED'):
             self.fail("The audit ended in unexpected state: %s!"
                       % finished_audit.get('state'))
 
-        _, action_plans = self.client.list_action_plans(
-            audit_uuid=audit['uuid'])
-        action_plan = action_plans['action_plans'][0]
+        return finished_audit
 
-        _, action_plan = self.client.show_action_plan(action_plan['uuid'])
-        created_actions = self.client.list_actions(
+    def get_action_plan_and_validate_actions(
+            self, audit_uuid, expected_action_types=None):
+        """Fetch action plan for a finished audit and validate actions.
+
+        Returns the action_plan object (dict).
+        """
+        _, action_plans = self.client.list_action_plans(
+            audit_uuid=audit_uuid)
+        action_plan_ref = action_plans['action_plans'][0]
+        _, action_plan = self.client.show_action_plan(
+            action_plan_ref['uuid'])
+
+        actions = self.client.list_actions(
             action_plan_uuid=action_plan["uuid"])[1]['actions']
 
-        if expected_actions:
-            action_types = {a['action_type'] for a in created_actions}
-            if set(expected_actions) != action_types:
-                self.fail("The audit has found action types %s when expecting "
-                          "%s" % (action_types, expected_actions))
+        if expected_action_types:
+            action_types = {a['action_type'] for a in actions}
+            if set(expected_action_types) != action_types:
+                self.fail(
+                    "The audit has found action types %s when expecting %s"
+                    % (action_types, expected_action_types)
+                )
 
-        if action_plan['state'] in ('SUPERSEDED', 'SUCCEEDED'):
-            # This means the action plan is superseded so we cannot trigger it,
-            # or it is empty.
-            return
+        return action_plan, actions
 
+    def execute_action_plan_and_validate_states(
+            self, action_plan_uuid,
+            expected_action_plan_states=['SUCCEEDED'],
+            expected_action_states=['SUCCEEDED']):
+        """Start the action plan execution and wait for completion.
+
+        Returns the finished action plan object (dict).
+        """
         # Execute the action by changing its state to PENDING
-        _, updated_ap = self.client.start_action_plan(action_plan['uuid'])
+        _, updated_ap = self.client.start_action_plan(action_plan_uuid)
 
         self.assertTrue(test_utils.call_until_true(
             func=functools.partial(
-                self.has_action_plan_finished, action_plan['uuid']),
+                self.has_action_plan_finished,
+                action_plan_uuid
+            ),
             duration=600,
             sleep_for=2
         ))
-        _, finished_ap = self.client.show_action_plan(action_plan['uuid'])
-        _, action_list = self.client.list_actions(
+        _, finished_ap = self.client.show_action_plan(action_plan_uuid)
+        _, finished_actions = self.client.list_actions(
             action_plan_uuid=finished_ap["uuid"])
         self.assertIn(updated_ap['state'], ('PENDING', 'ONGOING'))
-        self.assertIn(finished_ap['state'], ('SUCCEEDED', 'SUPERSEDED'))
+        self.assertIn(finished_ap['state'], expected_action_plan_states)
 
-        for action in action_list['actions']:
-            self.assertEqual('SUCCEEDED', action.get('state'))
+        for action in finished_actions['actions']:
+            self.assertIn(action.get('state'), expected_action_states)
+
+        return finished_ap, finished_actions
 
     def check_node_trait(self, node_id, trait):
         """Check if trait is in node traits
@@ -1070,6 +1133,47 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
                            for s in body.get('context', [])
                            if 'server_uuid' in s and 'node_hostname' in s]
 
+    def wait_for_instances_attributes_in_model(self, instances, attributes_map,
+                                               timeout=300):
+        """Waits until all instances have attributes updated in the model.
+
+        Builds a list of instances that have all attributes updated in the
+        model. Compare with the list of instances provided as argument.
+
+        :param instances: List of instances to wait for.
+        :param attributes_map: Map withattributes and expected values.
+        :param timeout: Timeout in seconds.
+
+        :raises: Exception if attributes were not updated in the model.
+        """
+
+        timeout_end = time.time() + timeout
+
+        _, body = self.client.list_data_models(data_model_type="compute")
+
+        # Get all server_uuid from model that have all attributes updated
+        # according to the attributes_map.
+        model_ids = [
+            s['server_uuid']
+            for s in body.get('context', [])
+            if 'server_uuid' in s and all(
+                s.get(k) == v for k, v in attributes_map.items())]
+
+        instance_ids = [i['id'] for i in instances]
+
+        # Check all instances are in the model list built.
+        while (not set(instance_ids) <= set(model_ids) or not model_ids):
+            time.sleep(15)
+            if time.time() >= timeout_end:
+                raise Exception("Attributes were not updated in the model.")
+
+            _, body = self.client.list_data_models(data_model_type="compute")
+            model_ids = [
+                s['server_uuid']
+                for s in body.get('context', [])
+                if 'server_uuid' in s and all(
+                    s.get(k) == v for k, v in attributes_map.items())]
+
     def wait_delete_instances_from_model(self, timeout=300):
         """Waits until all deleted instaces be removed from model."""
         timeout_end = time.time() + timeout
